=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2008-08-28 18:57:59 +0000
+++ bzrlib/branch.py	2008-09-15 14:27:19 +0000
@@ -2169,7 +2169,9 @@
         return self._get_bound_location(False)
 
     def get_stacked_on_url(self):
-        self._check_stackable_repo()
+        # you can always ask for the URL; but you might not be able to use it
+        # if the repo can't support stacking.
+        ## self._check_stackable_repo()
         stacked_url = self._get_config_location('stacked_on_location')
         if stacked_url is None:
             raise errors.NotStacked(self)

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2008-09-15 14:08:05 +0000
+++ bzrlib/remote.py	2008-09-15 14:27:19 +0000
@@ -600,8 +600,8 @@
         if isinstance(repository, RemoteRepository):
             raise AssertionError()
         self._real_repository = repository
-#        for fb in self._fallback_repositories:
-#            self._real_repository.add_fallback_repository(fb)
+        for fb in self._fallback_repositories:
+            self._real_repository.add_fallback_repository(fb)
         if self._lock_mode == 'w':
             # if we are already locked, the real repository must be able to
             # acquire the lock with our token.
@@ -733,9 +733,8 @@
         # on various RPC's.
         self._fallback_repositories.append(repository)
         # They are also seen by the fallback repository.  If it doesn't exist
-        # yet they'll be added then.
+        # yet they'll be added then.  This implicitly copies them.
         self._ensure_real()
-        self._real_repository.add_fallback_repository(repository)
 
     def add_inventory(self, revid, inv, parents):
         self._ensure_real()
@@ -1397,20 +1396,19 @@
         :raises UnstackableRepositoryFormat: If the repository does not support
             stacking.
         """
-        if False:
-            self._ensure_real()
-            return self._real_branch.get_stacked_on_url()
-        else:
-            try:
-                response = self._client.call('Branch.get_stacked_on_url',
-                    self._remote_path())
-            except errors.ErrorFromSmartServer, err:
-                # there may not be a repository yet, so we can't call through
-                # its _translate_error
-                _translate_error(err, branch=self)
+        try:
+            response = self._client.call('Branch.get_stacked_on_url',
+                self._remote_path())
             if response[0] != 'ok':
                 raise errors.UnexpectedSmartServerResponse(response)
             return response[1]
+        except errors.ErrorFromSmartServer, err:
+            # there may not be a repository yet, so we can't call through
+            # its _translate_error
+            _translate_error(err, branch=self)
+        except errors.UnknownSmartMethod, err:
+            self._ensure_real()
+            return self._real_branch.get_stacked_on_url()
 
     def lock_read(self):
         if not self._lock_mode:

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2008-09-15 14:08:05 +0000
+++ bzrlib/tests/test_remote.py	2008-09-15 14:27:19 +0000
@@ -563,6 +563,38 @@
         self.assertEqual(
             'file:///stacked/on', result)
 
+    def test_backwards_compatible(self):
+        # like with bzr1.6 with no Branch.get_stacked_on_url rpc
+        base_branch = self.make_branch('base', format='1.6')
+        stacked_branch = self.make_branch('stacked', format='1.6')
+        stacked_branch.set_stacked_on_url('../base')
+        client = FakeClient(self.get_url())
+        client.add_expected_call(
+            'BzrDir.open_branch', ('stacked/',),
+            'success', ('ok', ''))
+        client.add_expected_call(
+            'BzrDir.find_repositoryV2', ('stacked/',),
+            'success', ('ok', '', 'no', 'no', 'no'))
+        # called twice, once from constructor and then again by us
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('stacked/',),
+            'unknown', ('Branch.get_stacked_on_url',))
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('stacked/',),
+            'unknown', ('Branch.get_stacked_on_url',))
+        # this will also do vfs access, but that goes direct to the transport
+        # and isn't seen by the FakeClient.
+        bzrdir = RemoteBzrDir(self.get_transport('stacked'), _client=client)
+        branch = bzrdir.open_branch()
+        result = branch.get_stacked_on_url()
+        self.assertEqual('../base', result)
+        client.finished_test()
+        # it's in the fallback list both for the RemoteRepository and its vfs
+        # repository
+        self.assertEqual(1, len(branch.repository._fallback_repositories))
+        self.assertEqual(1,
+            len(branch.repository._real_repository._fallback_repositories))
+
     def test_get_stacked_on_real_branch(self):
         base_branch = self.make_branch('base', format='1.6')
         stacked_branch = self.make_branch('stacked', format='1.6')

